function [FOC,D,X,Y,L1_X,L2_X,L1_Y,L2_Y,Fevals] = foc_con_2D(m,h,Xmin,Xmax,Ymin,Ymax,c,param,glob)
%__________________________________________________________________________
% Use assign to convert m_X and m_Y into X,Y and Lagrange multipliers
m_X                     = m(1);
m_Y                     = m(2);
[X,L1_X,L2_X,dXdm_X]    = assign(m_X,Xmin,Xmax);
[Y,L1_Y,L2_Y,dYdm_Y]    = assign(m_Y,Ymin,Ymax);
%__________________________________________________________________________
% Derive set of 6 basis matrices we need to compute everything else
[B0_X,B1_X,B2_X]        = SM_splibas(glob.xgrid0,X,glob.D1_X,glob.D2_X,2);
[B0_Y,B1_Y,B2_Y]        = SM_splibas(glob.xgrid0,Y,glob.D1_Y,glob.D2_Y,2);
%__________________________________________________________________________
% Compute flow_XY which is guarenteed to be used below
flow_XY                 = flow(X,Y);
Fevals                  = 1;
%__________________________________________________________________________
% Boundaries for m_X
if m_X<0
    flow_XphY           = flow(X+h,Y);
    dFdX_c              = (1/h)*(flow_XphY - flow_XY) + dprod(B0_Y,B1_X)*c;     
    FOC1_X              = 2*m_X;                                                    % = dL1_X/dm_X
    Fevals              = Fevals + 1;
elseif m_X>1
    flow_XmhY           = flow(X-h,Y);
    dFdX_c              = (1/h)*(flow_XY - flow_XmhY) + dprod(B0_Y,B1_X)*c;
    FOC1_X              = -2*(m_X-1);                                               % = -dL2_X/dm_X
    Fevals              = Fevals + 1;
else
    flow_XmhY           = flow(X-h,Y);
    flow_XphY           = flow(X+h,Y);
    dFdX_c              = (1/(2*h))*(flow_XphY - flow_XmhY) + dprod(B0_Y,B1_X)*c;   % CENTERED
    FOC1_X              = (((1/(h^2))*(flow_XphY - 2*flow_XY + flow_XmhY)) + dprod(B0_Y,B2_X)*c)*dXdm_X;
    Fevals              = Fevals + 2;
end
%__________________________________________________________________________
% Boundaries for m_Y
if m_Y<0
    flow_XYph           = flow(X,Y+h);
    dFdY_c              = (1/h)*(flow_XYph - flow_XY) + dprod(B1_Y,B0_X)*c;         % RIGHT
    FOC2_Y              = 2*m_Y;                                                    % = dL1_Y/dm_Y
    Fevals              = Fevals + 1;
elseif m_Y>1
    flow_XYmh           = flow(X,Y-h);
    dFdY_c              = (1/h)*(flow_XY - flow_XYmh) + dprod(B1_Y,B0_X)*c;         % LEFT
    FOC2_Y              = -2*(m_Y-1);                                               % = -dL2_Y/dm_Y
    Fevals              = Fevals + 1;
else
    flow_XYmh           = flow(X,Y-h);
    flow_XYph           = flow(X,Y+h);
    dFdY_c              = (1/(2*h))*(flow_XYph - flow_XYmh)+ dprod(B1_Y,B0_X)*c;    % CENTERED
    FOC2_Y              = (((1/(h^2))*(flow_XYph - 2*flow_XY + flow_XYmh)) + dprod(B2_Y,B0_X)*c)*dYdm_Y;    % = F_{Y,Y}(X(m_x),Y(m_y)/dY * dY/dm_Y
    Fevals              = Fevals + 2;
end
FOC     = [ dFdX_c + L1_X - L2_X;   
            dFdY_c + L1_Y - L2_Y]  ; 
%__________________________________________________________________________
% Compute cross-partial derivatives with respect to m_X and m_Y
%__________________________________________________________________________
if m_X<0 && (m_Y>=0 && m_Y<=1)
    flow_XphYph = flow(X+h,Y+h);
    flow_XphYmh = flow(X+h,Y-h);
    flow_XY     = (1/(2*h^2))*(flow_XphYph - flow_XYph - flow_XphYmh + flow_XYmh);
    cont_XY     = dprod(B1_Y,B1_X)*c;
    FOC1_Y      = dYdm_Y*(flow_XY + cont_XY);
    FOC2_X      = 0;
end
%__________________________________________________________________________
if m_X>1 && (m_Y>=0 && m_Y<=1)
    flow_XmhYph = flow(X-h,Y+h);
    flow_XmhYmh = flow(X-h,Y-h);
    flow_XY     = (1/(2*h^2))*(flow_XYph - flow_XmhYph - flow_XYmh + flow_XmhYmh);
    cont_XY     = dprod(B1_Y,B1_X)*c;
    FOC1_Y      = dYdm_Y*(flow_XY + cont_XY);
    FOC2_X      = 0;
end
%__________________________________________________________________________
if m_Y<0 && (m_X>=0 && m_X<=1)
    flow_XphYph = flow(X+h,Y+h);
    flow_XmhYph = flow(X-h,Y+h); 
    flow_XY     = (1/(2*h^2))*(flow_XphYph - flow_XphY - flow_XmhYph + flow_XmhY);
    cont_XY     = dprod(B1_Y,B1_X)*c;
    FOC1_Y      = 0;
    FOC2_X      = dXdm_X*(flow_XY + cont_XY);
end
%__________________________________________________________________________
if m_Y>1 && (m_X>=0 && m_X<=1)
    flow_XphYmh = flow(X+h,Y-h);
    flow_XmhYmh = flow(X-h,Y-h);
    flow_XY     = (1/(2*h^2))*(flow_XphY - flow_XphYmh - flow_XmhY + flow_XmhYmh);
    cont_XY     = dprod(B1_Y,B1_X)*c;
    FOC1_Y      = 0;
    FOC2_X      = dXdm_X*(flow_XY + cont_XY);
end
%__________________________________________________________________________
if (m_X<0 || m_X>1) && (m_Y<0 || m_Y>1)
   FOC1_Y       = 0;
   FOC2_X       = 0;
end
%__________________________________________________________________________
if m_X>=0 && m_X<=1 && m_Y>=0 && m_Y<=1
    flow_XphYph = flow(X+h,Y+h);
	flow_XmhYmh = flow(X-h,Y-h);
    cont_XY     = dprod(B1_Y,B1_X)*c;
    flow_XY     = (1/(4*h^2))*(flow_XphYph - flow_XphY - flow_XYph + 2*flow_XY - flow_XmhY - flow_XYmh + flow_XmhYmh);
    FOC1_Y      = dYdm_Y*(flow_XY + cont_XY);
    FOC2_X      = dXdm_X*(flow_XY + cont_XY);
end
%__________________________________________________________________________
D               = [ FOC1_X, FOC1_Y ;
                    FOC2_X, FOC2_Y ];
%__________________________________________________________________________
%% NESTED FUNCTIONS (will need to be specified when solving problem)
    
    function out = flow(X,Y)
        [~,out]   = F_2D(X,Y,c,param,glob,1);
    end

end  















